/*************************************************************
 * remc_tx_main.c
 *
 * REMC transmit main module
 *
 * Copyright (C)SEIKO EPSON CORPORATION 2010. All rights reserved.
 ************************************************************/

#include <stdio.h>
#include <s1c176xx.h>

#ifdef	PE_REMC_TX
#include <remc_api.h>
#include <mux_api.h>

#define	TX_NUMBER						6								// TX/RX number

// carrier parameter
#define	REMCTX_MAIN_CARRIER_H			54U
#define	REMCTX_MAIN_CARRIER_L			54U

#define	REMCTX_MAIN_SENDDATA_NUM		10U	// send data number

// REMC send data status
#define	REMCTX_MAIN_STATUS_IDLE			0	// idle
#define	REMCTX_MAIN_STATUS_HIGH			1	// sending high
#define	REMCTX_MAIN_STATUS_LOW			2	// sending low
typedef struct tag_REMC_MAIN_TX_PARAM {
	unsigned char stateTx;					// send data state
	unsigned char *dataTx;					// send data buffer
} REMC_MAIN_TX_PARAM;
volatile static REMC_MAIN_TX_PARAM paramTx;

// prototype
void remcTxMain(void);
static void startREMCTx(void);
static void stopREMCTx(void);
static unsigned char sendData(unsigned char *buf, unsigned char len);
void remcIntHandler(void)__attribute__((interrupt_handler));

extern void remcRxUnderFlowIntrHandler(void);
extern void remcRxFallIntrHandler(void);
extern void remcRxRiseIntrHandler(void);


/*
 * REMC tx sample program
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void remcTxMain(void)
{
	unsigned char	data[REMCTX_MAIN_SENDDATA_NUM] =
		{5, 10, 15, 20, 25, 30, 35, 40, 45, 50};		// send data

	printf("<<< REMC transmit demonstration start >>>\n");

	// start remc
	startREMCTx();

	// send data
	sendData(&data[0], REMCTX_MAIN_SENDDATA_NUM);

	// stop remc
	stopREMCTx();

	// clear port
	MUX_setREMCport(MUX_OFF);

	printf("<<< REMC transmit demonstration finish >>>\n");
}

/*
 * start REMC for data send.
 * param	void
 * return	void
 * history	:	2009/12/25
 */
static void startREMCTx(void)
{
	REMC_INIT_PARAM param;												// initialize parameter

	param.selectCarrier = REMC_CLK2;
	param.selectLength = REMC_CLK1024;
	param.carrireL = REMCTX_MAIN_CARRIER_L;
	param.carrireH = REMCTX_MAIN_CARRIER_H;

	paramTx.stateTx = REMCTX_MAIN_STATUS_IDLE;
	paramTx.dataTx = 0;
	REMC_init(&param);

	REMC_initInt(REMC_INT_LEVEL3);
	REMC_resetIntFlag(REMC_UDFINT_RESET | REMC_RISINGINT_RESET | REMC_FALLINGINT_RESET);
	REMC_setEnable(REMC_TRANSMIT, REMC_ENABLE);
}

/*
 * stop REMC for data send.
 * param	void
 * return	void
 * history	:	2009/12/25
 */
static void stopREMCTx(void)
{
	REMC_setEnable(REMC_TRANSMIT, REMC_DISABLE);
	REMC_controlInt(REMC_UDFINT_DISABLE | REMC_RISINGINT_DISABLE | REMC_FALLINGINT_DISABLE);
	REMC_initInt(REMC_INT_LEVEL0);
}

/*
 * send data.
 * param	unsigned char  *buf      send data buffler
 * param	unsigned char  len       send data length
 * return	unsigned char            actual send data length
 * history	:	2009/12/25
 */
static unsigned char sendData(unsigned char *buf, unsigned char len)
{
	unsigned char	data;												// send 1byte data
	unsigned char	counter;											// bit counter for 1byte
	unsigned char	actual;												// actual send data length

	paramTx.dataTx = buf;
	paramTx.stateTx = REMCTX_MAIN_STATUS_IDLE;

	// interrupt enable
	REMC_controlInt(REMC_UDFINT_ENABLE | REMC_RISINGINT_DISABLE | REMC_FALLINGINT_DISABLE);

	// send data
	actual = 0;
	while(actual < len) {
		data = *paramTx.dataTx;
		counter = 0;
		printf("Transmitted data : %02d\n", *paramTx.dataTx);

		for(counter = 0; counter < TX_NUMBER; counter++) {
			paramTx.stateTx = REMCTX_MAIN_STATUS_HIGH;
			REMC_setTransmitData(REMC_DATA_H, data);

			while(paramTx.stateTx != REMCTX_MAIN_STATUS_IDLE) {
				;
			}
		}
		paramTx.dataTx++;
		actual++;
	}
	// interrupt disable
	REMC_controlInt(REMC_UDFINT_DISABLE | REMC_RISINGINT_DISABLE | REMC_FALLINGINT_DISABLE);

	return actual;
}

/*
 * REMC interrupt handler
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void remcIntHandler(void)
{
	unsigned short	flgIntr;		// interrupt
	unsigned short	flgClearIntr;	// clear flag

	REMC_checkIntFlag(&flgIntr);

	flgClearIntr = 0;

	// under flow
	if((flgIntr & REMC_UDFINT_OCCURRED) == REMC_UDFINT_OCCURRED) {
		if(paramTx.stateTx == REMCTX_MAIN_STATUS_HIGH) {
			// for data transmit
			REMC_setTransmitData(REMC_DATA_L, *paramTx.dataTx);
			paramTx.stateTx = REMCTX_MAIN_STATUS_LOW;		// send low
		} else if(paramTx.stateTx == REMCTX_MAIN_STATUS_LOW) {
			// for data transmit
			paramTx.stateTx = REMCTX_MAIN_STATUS_IDLE;		// stop transmit
		} else {
			// for data receive
			remcRxUnderFlowIntrHandler();			// if status is IDLE, it means receive data.
		}
		flgClearIntr |= REMC_UDFINT_RESET;
	}

	// rising edge(for data receive)
	if((flgIntr & REMC_RISINGINT_OCCURRED) == REMC_RISINGINT_OCCURRED) {
		remcRxFallIntrHandler();					// call Rx function
		flgClearIntr |= REMC_RISINGINT_RESET;
	}

	// falling edge(for data receive)
	if((flgIntr & REMC_FALLINGINT_OCCURRED) == REMC_FALLINGINT_OCCURRED) {
		remcRxRiseIntrHandler();					// call Rx function
		flgClearIntr |= REMC_FALLINGINT_RESET;
	}

	REMC_resetIntFlag(flgClearIntr);
}
#endif	// end of PE_REMC_TX
