/*************************************************************
 * svd_main.c
 *
 * SVD main module
 *
 * Copyright (C)SEIKO EPSON CORPORATION 2009. All rights reserved.
 ************************************************************/

#include <stdio.h>
#include <string.h>
#include <s1c176xx.h>

#ifdef	PE_SVD
#include <svd_api.h>
#include <osc_api.h>
#include <t16_api.h>

/* flag value */
#define SVD_MAIN_STOP 			0		// SVD is stopped
#define SVD_MAIN_RUN 			1		// SVD is running
#define SVD_MAIN_SVD_DETECT 	2		// SVD interrupt is occured

/* compare voltage parameters */
typedef struct tag_SVD_MAIN_VOLT_PARAM {
	unsigned char *string;			// string
	unsigned char compareData;		// compare voltage data
} SVD_MAIN_VOLT_PARAM;

static SVD_MAIN_VOLT_PARAM paramTable[SVD_COMPVOL_MAX] =
{
	{"    ", SVD_COMPVOL_RSV},
	{"1.7V", SVD_COMPVOL_18V},
	{"1.8V", SVD_COMPVOL_19V},
	{"1.9V", SVD_COMPVOL_20V},
	{"2.0V", SVD_COMPVOL_21V},
	{"2.1V", SVD_COMPVOL_22V},
	{"2.2V", SVD_COMPVOL_23V},
	{"2.3V", SVD_COMPVOL_24V},
	{"2.4V", SVD_COMPVOL_25V},
	{"2.5V", SVD_COMPVOL_26V},
	{"2.6V", SVD_COMPVOL_27V},
	{"2.7V", SVD_COMPVOL_28V},
	{"2.8V", SVD_COMPVOL_29V},
	{"2.9V", SVD_COMPVOL_30V},
	{"3.0V", SVD_COMPVOL_31V},
	{"3.1V", SVD_COMPVOL_32V}
};

static volatile unsigned char	voltageIndex;		// compare voltage index
static volatile unsigned char	flgSVDRunStatus;	// SVD program running status flag

// prototype
void svdMain(void);
static void startT16(unsigned char chNo);
static void stopT16(unsigned char chNo);
static void startSVD(void);
static void stopSVD(void);
void svdIntHandler(void)__attribute__((interrupt_handler));
void svdT16IntHandler(void);

extern void t16SetCallBackFunc(void(*name)(void));
extern void t16ClearCallBackFunc(void);

/*
 * SVD sample program
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void svdMain(void)
{
	printf("<<< SVD demonstration start >>>\n");

	// set SVD clock
	CLG_OSC_setSVDClock(CLG_OSC_SVDCLK_OSC1);
	CLG_OSC_controlSVD(CLG_OSC_SVD_ENABLE);

	// start SVD
	voltageIndex = SVD_COMPVOL_18V;
	flgSVDRunStatus = SVD_MAIN_RUN;
	startSVD();

	// set a call back function to T16 interrupt handler
	t16SetCallBackFunc(&svdT16IntHandler);

	// start T16
	startT16(T16_CH0);

	while(flgSVDRunStatus == SVD_MAIN_RUN) {
		;
	}

	// display result
	if(flgSVDRunStatus == SVD_MAIN_SVD_DETECT) {
		printf("Vdd=%s\n", paramTable[voltageIndex].string);
	} else {
		printf("SVD interrupt did not occur.\n");
	}

	// stop T16
	stopT16(T16_CH0);

	// unregister a call back function from T16 timer handler
	t16ClearCallBackFunc();

	// stop SVD
	stopSVD();

	// stop SVD clock
	CLG_OSC_controlSVD(CLG_OSC_SVD_DISABLE);

	printf("<<< SVD demonstration finish >>>\n");
}

/*
 * start T16 timer for 1 sec interval.
 * param	T16 channel number.
 * return	void
 * history	:	2009/12/25
 */
static void startT16(unsigned char chNo)
{
	T16_TIM_MODE	mode;	// timer mode setting

	mode.ckactv	= T16_TIMMODE_ACTVL;
	mode.cksl	= T16_TIMMODE_INTERNAL;
	mode.trmd	= T16_TIMMODE_REPEAT;
	T16_init(chNo, T16_CNTCLK_1, &mode, OSC3_CLK/16384);

	T16_initInt(chNo, T16_INT_LEVEL3);
	T16_controlInt(chNo, T16_INT_ENABLE);
	T16_setTimerRun(chNo, T16_TIMER_RUN);
}

/*
 * stop T16 timer.
 * param	T16 channel number.
 * return	void
 * history	:	2009/12/25
 */
static void stopT16(unsigned char chNo)
{
	T16_setTimerRun(chNo, T16_TIMER_STOP);
	T16_controlInt(chNo, T16_INT_DISABLE);
	T16_initInt(chNo, T16_INT_LEVEL0);
}

/*
 * start SVD
 * param	void
 * return	void
 * history	:	2009/12/25
 */
static void startSVD(void)
{
	unsigned short	i;													// wait counter

	SVD_setCompareVoltage(paramTable[voltageIndex].compareData);
	SVD_controlDetection(SVD_DETECT_ENABLE);

	// wait for the stability of SVD.   500us or more
	for ( i=0; i<0x500; i++ )
	{
		asm ( "nop" );
	}
	SVD_initInt(SVD_INT_LEVEL3);
	SVD_controlInt(SVD_INT_ENABLE);
}

/*
 * stop SVD
 * param	void
 * return	void
 * history	:	2009/12/25
 */
static void stopSVD(void)
{
	SVD_controlInt(SVD_INT_DISABLE);
	SVD_initInt(SVD_INT_LEVEL0);
	SVD_controlDetection(SVD_DETECT_DISABLE);
}

/*
 * SVD interrupt handler
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void svdIntHandler(void)
{
	unsigned char flgIntr;				//check interrupt flag
	unsigned char svdResult;			//sdv result

	// check interrupt flag
	SVD_checkIntFlag(&flgIntr);

	// reset interrupt flag
	SVD_resetIntFlag();

	if((flgIntr & SVD_INT_OCCURRED) == SVD_INT_OCCURRED) {
		SVD_getDetectionResult(&svdResult);

		flgSVDRunStatus = SVD_MAIN_SVD_DETECT;
	}
}

/*
 * T16 interrupt handler.
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void svdT16IntHandler(void)
{
	// reset interrupt flag
	T16_resetIntFlag(T16_CH0);

	if(voltageIndex != SVD_COMPVOL_32V) {
		voltageIndex++;
	} else {
		voltageIndex = SVD_COMPVOL_18V;
		flgSVDRunStatus = SVD_MAIN_STOP;
	}
	SVD_setCompareVoltage(paramTable[voltageIndex].compareData);
}


#endif	// end of PE_SVD
