/*************************************************************
 * t16a2_main.c
 *
 * PWM timer(T16A2) main module
 *
 * Copyright (C)SEIKO EPSON CORPORATION 2010. All rights reserved.
 ************************************************************/

#include <stdio.h>
#include <s1c176xx.h>

#ifdef PE_T16A2
#include <osc_api.h>
#include <mux_api.h>
#include <t16a2_api.h>

#define	T16A2_MAIN_LOOP_CT_MAX			5								// loop counter max
#define	T16A2_MAIN_A					0x00							// compare A index
#define	T16A2_MAIN_B					T16A2_MAIN_A + 1				// compare B index
#define	T16A2_MAIN_CMP_MATCH_NO			T16A2_MAIN_B + 1				// a number of compare match registers
#define	T16A2_MAIN_CH					T16A2_CH0						// T16A2 ch

static unsigned short	t16a2BufCounter[T16A2_MAIN_CMP_MATCH_NO][T16A2_MAIN_LOOP_CT_MAX];	// counter record buffer
static volatile unsigned char	t16a2LoopCt;							// loop counter

static void (*callBackFunction)(void) = 0;								// call back function

/* prototype */
void t16a2Main(void);
static void setT16A2Clock(unsigned char chNo);
static void startNormalT16A2(unsigned char chNo);
static void startHalfT16A2(unsigned char chNo);
static void stopT16A2(unsigned char chNo);
void t16a2IntHandler(void)__attribute__((interrupt_handler));
void t16a2SetCallBackFunc(void(*name)(void));
void t16a2ClearCallBackFunc(void);


/*
 * T16A2 timer sample program
 * param	void
 * return	void
 * history	:	2010/04/05
 */
void t16a2Main(void)
{
	MUX_USE_T16A2PORT	use;											// T16A2 timer use port

	printf("<<< PWM timer(T16A2) demonstration start >>>\n");

	// set T16A2 port
	use.useEXCL = MUX_EXCL_OFF;
	use.useToutA = MUX_TOUTA_ON;
	use.useToutB = MUX_TOUTB_OFF;
#ifdef S1C17624
	use.useEXCLS = MUX_EXCLXS_OFF;
	use.useToutAPort = MUX_TOUTA5P_CH0_P31;
	use.useToutBPort = MUX_TOUTBXP_OFF;
#endif // end of S1C17624
	MUX_setT16A2port(T16A2_CH0, &use, MUX_ON);

	// T16A2 clock enable
	setT16A2Clock(T16A2_MAIN_CH);

	// start T16A2 timer(normal clock mode)
	startNormalT16A2(T16A2_MAIN_CH);

	t16a2LoopCt = 0;
	while(t16a2LoopCt < T16A2_MAIN_LOOP_CT_MAX) {
		asm("halt");
		asm("nop");

		t16a2LoopCt++;
	}

	// stop T16A2 timer
	stopT16A2(T16A2_MAIN_CH);

	t16a2LoopCt = 0;
	printf("Normal clock mode start\n");
	while(t16a2LoopCt < T16A2_MAIN_LOOP_CT_MAX) {
		t16a2LoopCt++;
		printf("*** PWM compare A interrupt : %4d ***\n",
				t16a2BufCounter[T16A2_MAIN_A][t16a2LoopCt - 1U]);
		printf("*** PWM compare B interrupt : %4d ***\n",
				t16a2BufCounter[T16A2_MAIN_B][t16a2LoopCt - 1U]);
	}

	// start T16A2 timer(half clock mode)
	startHalfT16A2(T16A2_MAIN_CH);

	t16a2LoopCt = 0;
	while(t16a2LoopCt < T16A2_MAIN_LOOP_CT_MAX) {
		asm("halt");
		asm("nop");

		t16a2LoopCt++;
	}

	// stop T16A2 timer
	stopT16A2(T16A2_MAIN_CH);

	t16a2LoopCt = 0;
	printf("Half clock mode start\n");
	while(t16a2LoopCt < T16A2_MAIN_LOOP_CT_MAX) {
		t16a2LoopCt++;
		printf("*** PWM compare B interrupt : %4d ***\n",
				t16a2BufCounter[T16A2_MAIN_B][t16a2LoopCt - 1U]);
	}

	// disable T16A2 clock
	CLG_OSC_controlT16A2(T16A2_MAIN_CH, CLG_OSC_T16A2CLK_ENABLE);

	// clear port
	use.useEXCL = MUX_EXCL_OFF;
	use.useToutA = MUX_TOUTA_ON;
	use.useToutB = MUX_TOUTB_OFF;
#ifdef S1C17624
	use.useEXCLS = MUX_EXCLXS_OFF;
	use.useToutAPort = MUX_TOUTA5P_CH0_P31;
	use.useToutBPort = MUX_TOUTBXP_OFF;
#endif // end of S1C17624
	MUX_setT16A2port(T16A2_CH0, &use, MUX_OFF);

	printf("<<< PWM timer(T16A2) demonstration finish >>>\n");
}


/*
 * start T16A2 clock
 * param	chNo						T16A2 channel number.
 * return	void
 * history	:	2010/04/05
 */
static void setT16A2Clock(unsigned char chNo)
{
	CLG_OSC_SET_T16A2	select;											// T16A2 clock
	select.clkdiv=CLG_OSC_T16A2RATIO_16384;
	select.clksrc=CLG_OSC_T16A2CLK_OSC3;
	select.multimd = CLG_OSC_T16A2MD_NORMAL;

	// set T16A2 clock mode
	CLG_OSC_setT16A2Clock(chNo, &select);

	// enable T16A2 clock
	CLG_OSC_controlT16A2(chNo, CLG_OSC_T16A2CLK_ENABLE);
}


/*
 * start T16A2 timer(normal clock mode).
 * param	chNo						T16A2 channel number.
 * param	clkMd						T16A2 clock mode normal/half
 * return	void
 * history	:	2010/04/05
 */
static void startNormalT16A2(unsigned char chNo)
{
	T16A2_INIT_PARAM	param;												// timer init parameter

	// set comparator/capture parameter
	param.cca.captrg	= T16A2_CAPTRG_NONE;
	param.cca.toutmd	= T16A2_TOUT_AR_BF;
	param.cca.toutinv	= T16A2_TOUT_NORMAL;
	param.cca.ccmd		= T16A2_MODE_COMPARATOR;
	param.ccb.captrg	= T16A2_CAPTRG_NONE;
	param.ccb.toutmd	= T16A2_TOUT_OFF;
	param.ccb.toutinv	= T16A2_TOUT_NORMAL;
	param.ccb.ccmd		= T16A2_MODE_COMPARATOR;

	//set timer mode
	param.mode.hcm		= T16A2_HLFCLK_DISABLE;
	param.mode.ccabcnt	= T16A2_MAIN_CH;
	param.mode.cbufen	= T16A2_CMPBUF_ENABLE;
	param.mode.trmd		= T16A2_CNTMODE_REPEAT;

	// set input clock
	param.clock			= T16A2_CLKSRC_OSC3;

	// set counter data
	param.dataA	= OSC3_CLK/16384/4;
	param.dataB	= OSC3_CLK/16384/2;

#ifdef S1C17624
	// set T16A2 port
	param.useEXCLS = MUX_EXCLXS_OFF;
	param.useToutAPort = MUX_TOUTA5P_CH0_P31;
	param.useToutBPort = MUX_TOUTBXP_OFF;
#endif // end of S1C17624

	T16A2_init(chNo, &param);
	T16A2_initInt(chNo, T16A2_INT_LEVEL3);
	T16A2_controlInt(chNo, (T16A2_CAPBOVW_DISABLE | T16A2_CAPAOVW_DISABLE |
			T16A2_CAPBINT_DISABLE | T16A2_CAPAINT_DISABLE | T16A2_CMPBINT_ENABLE | T16A2_CMPAINT_ENABLE));
	T16A2_setTimerRun(chNo, T16A2_TIMER_RUN);
}


/*
 * start T16A2 timer(half clock mode).
 * param	chNo						T16A2 channel number.
 * return	void
 * history	:	2010/04/05
 */
static void startHalfT16A2(unsigned char chNo)
{
	T16A2_INIT_PARAM	param;												// timer init parameter

	// set comparator/capture parameter
	param.cca.captrg	= T16A2_CAPTRG_NONE;
	param.cca.toutmd	= T16A2_TOUT_AR_BF;
	param.cca.toutinv	= T16A2_TOUT_NORMAL;
	param.cca.ccmd		= T16A2_MODE_COMPARATOR;
	param.ccb.captrg	= T16A2_CAPTRG_NONE;
	param.ccb.toutmd	= T16A2_TOUT_OFF;
	param.ccb.toutinv	= T16A2_TOUT_NORMAL;
	param.ccb.ccmd		= T16A2_MODE_COMPARATOR;

	//set timer mode
	param.mode.hcm		= T16A2_HLFCLK_ENABLE;
	param.mode.ccabcnt	= T16A2_MAIN_CH;
	param.mode.cbufen	= T16A2_CMPBUF_ENABLE;
	param.mode.trmd		= T16A2_CNTMODE_REPEAT;

	// set input clock
	param.clock	= T16A2_CLKSRC_OSC3;

	// set counter data
	param.dataA	= OSC3_CLK/16384/4;
	param.dataB	= OSC3_CLK/16384/2;

#ifdef S1C17624
	// set T16A2 port
	param.useEXCLS = MUX_EXCLXS_OFF;
	param.useToutAPort = MUX_TOUTA5P_CH0_P31;
	param.useToutBPort = MUX_TOUTBXP_OFF;
#endif // end of S1C17624

	T16A2_init(chNo, &param);
	T16A2_initInt(chNo, T16A2_INT_LEVEL3);
	T16A2_controlInt(chNo, (T16A2_CAPBOVW_DISABLE | T16A2_CAPAOVW_DISABLE |
			T16A2_CAPBINT_DISABLE | T16A2_CAPAINT_DISABLE | T16A2_CMPBINT_ENABLE | T16A2_CMPAINT_DISABLE));
	T16A2_setTimerRun(chNo, T16A2_TIMER_RUN);
}

/*
 * stop T16A2 timer.
 * param	chNo						T16A2 channel number.
 * return	void
 * history: 2010/04/05
 */
static void stopT16A2(unsigned char chNo)
{
	T16A2_setTimerRun(chNo, T16A2_TIMER_STOP);
	T16A2_controlInt(chNo, (T16A2_CAPBOVW_DISABLE | T16A2_CAPAOVW_DISABLE |
			T16A2_CAPBINT_DISABLE | T16A2_CAPAINT_DISABLE | T16A2_CMPBINT_DISABLE | T16A2_CMPAINT_DISABLE));
	T16A2_initInt(chNo, T16A2_INT_LEVEL0);
}


/*
 * T16A2 interrupt handler
 * param	void
 * return	void
 * history	:	2010/04/05
 */
void t16a2IntHandler(void)
{
	unsigned short	intFlag;											// interrupt flag

	if(callBackFunction != 0) {
		callBackFunction();												// call back
	} else {
		// get interrupt status
		T16A2_checkIntFlag(T16A2_MAIN_CH, &intFlag);

		// compare A interrupt
		if((intFlag & T16A2_CMPAINT_OCCURRED) == T16A2_CMPAINT_OCCURRED) {
			T16A2_getCounterData(T16A2_MAIN_CH, &t16a2BufCounter[T16A2_MAIN_A][t16a2LoopCt]);
		}

		// compare B interrupt
		if((intFlag & T16A2_CMPBINT_OCCURRED) == T16A2_CMPBINT_OCCURRED) {
			T16A2_getCounterData(T16A2_MAIN_CH, &t16a2BufCounter[T16A2_MAIN_B][t16a2LoopCt]);
		}

		// reset interrupt flag
		T16A2_resetIntFlag(T16A2_MAIN_CH, intFlag);
	}
}

/*
 * set interrupt call back function.
 * param	void(*name)(void)							call back function address
 * return	void
 * history	:	2010/04/05
 */
void t16a2SetCallBackFunc(void(*name)(void))
{
	callBackFunction = name;
}

/*
 * clear interrupt call back function.
 * param	void
 * return	void
 * history	:	2010/04/05
 */
void t16a2ClearCallBackFunc(void)
{
	callBackFunction = 0;
}


#endif // end of PE_T16A2





