/**
 * @file	i2cm.c
 * @brief	I2C master driver
 * 
 * - Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_I2CM
#include <i2cm_api.h>
#include <mux_api.h>

/*
 * definition of "define"
 */
#define	SLV_A7A0							0x00ff							// slave address[7:0]
#define	SLV_A9A8							0x0300							// slave address[9:8]
#define	SLV_10BIT_MODE						0x00f0							// 10-bit address mode data

#define	SLV_07BIT_ADDRESS					0x007f							// 7-bit address range
#define	SLV_10BIT_ADDRESS					0x03ff							// 10-bit address range


/*
 * definition of constants
 */


/*
 * Declarations of function prototype
 */
short I2CM_init(unsigned short selectBit);
short I2CM_setComEnable(unsigned short selectBit);
short I2CM_genCondition(unsigned short selectBit);
unsigned short I2CM_checkTransmitReg(void);
void I2CM_setTransmitData(unsigned char data);
unsigned short I2CM_checkTransmitBusy(void);
unsigned short I2CM_getSlaveResponse(void);
short I2CM_setReceiveStart(unsigned short response);
unsigned short I2CM_checkReceiveBusy(void);
void I2CM_getReceiveData(unsigned char *data);
unsigned short I2CM_checkReceiveReg(void);
short I2CM_initInt(unsigned char intLevel);
short I2CM_controlInt(unsigned short selectBit);
short I2CM_transmitSlaveAddress(unsigned char selectBit, unsigned short dir, unsigned short *address);


/**
 * @brief	I2C master initialize.
 * - 1. check parameters.
 * - 2. disable I2C master.
 * - 3. set I2C master ports.
 * - 4. set a noise removal function.
 *
 * @param	selectBit; unsigned short: 		I2C master initialize parameter
 * -		I2CM_NOISEREMOVE_OFF : noise removal function off
 * -		I2CM_NOISEREMOVE_ON  : noise removal function on
 * @return	short:							end status
 * -		I2CM_SUCCESS : success
 * -		I2CM_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short I2CM_init(unsigned short selectBit)
{
	// parameter check
	if(selectBit > I2CM_NOISEREMOVE_ON) {
		return	I2CM_INVALID_PARAMETER;
	}
	
	// I2C master disable
	I2CM_setComEnable(I2CM_CTL_DISABLE);
	
	// set I2C master port
	MUX_setI2CMport(MUX_I2CM_0, MUX_ON);
	
	// set initialize parameter
	RegModify(R16_I2CM_CTL, NSERM, (selectBit << NSERM_SFT));
	
	return	I2CM_SUCCESS;
}

/**
 * @brief	set an I2C master enable / disable.
 * - 1. check parameters.
 * - 2. set an I2C master enable/disable.
 *
 * @param	selectBit; unsigned short:		I2C master enable / disable
 * -		I2CM_CTL_DISABLE : disable
 * -		I2CM_CTL_ENABLE  : enable
 * @return	short:							end status
 * -		I2CM_SUCCESS : success
 * -		I2CM_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short I2CM_setComEnable(unsigned short selectBit)
{
	// parameter check
	if(selectBit > I2CM_CTL_ENABLE) {
		return	I2CM_INVALID_PARAMETER;
	}
	
	// I2C master enable / disable
	RegWrite(R16_I2CM_EN, selectBit);
	
	return	I2CM_SUCCESS;
}

/**
 * @brief	generate a start / stop condition.
 * - 1. check parameters.
 * - 2. generate a start/stop condition.
 *
 * @param	selectBit; unsigned short:		select start / stop condition
 * -		I2CM_CONDITION_START : generate a start condition
 * -		I2CM_CONDITION_STOP  : generate a stop condition
 * @return	short:							end status
 * -		I2CM_SUCCESS : success
 * -		I2CM_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short I2CM_genCondition(unsigned short selectBit)
{
	// parameter check
	if(selectBit > I2CM_CONDITION_MASK) {
		return	I2CM_INVALID_PARAMETER;
	}
	
	// start / stop condition generation
	RegModify(R16_I2CM_CTL, I2CM_CONDITION_MASK, (selectBit << STRT_SFT));
	
	return	I2CM_SUCCESS;
}

/**
 * @brief	check a transmit data register.
 * - 1. check a transmit data register condition.
 *
 * @return	unsigned short:					transmit data register condition
 * -		I2CM_TXREG_EMPTY    : transmit data register is empty
 * -		I2CM_TXREG_NOTEMPTY : transmit data register is not empty
 * @date	2009/12/25
 */
unsigned short I2CM_checkTransmitReg(void)
{
	// transmit data register check
	return	(unsigned short)((RegRead(R16_I2CM_DAT) & TXE) >> TXE_SFT);
}

/**
 * @brief	set a transmit data.
 * - 1. set a transmit data.
 *
 * @param	data; unsigned char:			transmit data
 * -		from 0x00 to 0xFF
 * @return	none
 * @date	2009/12/25
 */
void I2CM_setTransmitData(unsigned char data)
{
	// transmit data & start set
	RegModify(R16_I2CM_DAT, I2CM_TX_MASK, (TXE | ((unsigned short)data << RTDT_SFT)));
}

/**
 * @brief	check a transmit BUSY flag.
 * - 1. check a transmit BUSY flag.
 *
 * @return	unsigned short:					transmit condition
 * -		I2CM_TX_IDLE : transmit is idle
 * -		I2CM_TX_BUSY : transmit is busy
 * @date	2009/12/25
 */
unsigned short I2CM_checkTransmitBusy(void)
{
	// tranmit busy flag check
	return	(unsigned short)((RegRead(R16_I2CM_CTL) & TBUSY) >> TBUSY_SFT);
}

/**
 * @brief	get a slave response.
 * - 1. get a response of the slave after 1-byte data sending.
 *
 * @return	unsigned short:					slave response
 * -		I2CM_SLAVE_ACK : response ACK
 * -		I2CM_SLAVE_NAK : response NAK
 * @date	2009/12/25
 */
unsigned short I2CM_getSlaveResponse(void)
{
	// slave response get
	return	(unsigned short)((RegRead(R16_I2CM_DAT) & RTACK) >> RTACK_SFT);
}

/**
 * @brief	set a start 1-byte data receiving.
 * - 1. check parameters.
 * - 2. set a response data.
 * - 3. set a start 1-byte data receiving.
 *
 * @param	response; unsigned short:		response data
 * -		I2CM_SLAVE_ACK : response data ACK
 * -		I2CM_SLAVE_NAK : response data NAK
 * @return	short:							end status
 * -		I2CM_SUCCESS : success
 * -		I2CM_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short I2CM_setReceiveStart(unsigned short response)
{
	// parameter check
	if(response > I2CM_SLAVE_NAK) {
		return	I2CM_INVALID_PARAMETER;
	}
	
	// response data & receive start set
	RegModify(R16_I2CM_DAT, I2CM_RX_MASK, (RXE | (response << RTACK_SFT)));
	
	return	I2CM_SUCCESS;
}

/**
 * @brief	check a receive BUSY flag.
 * - 1. check a receive BUSY flag.
 *
 * @return	unsigned short:					receive condition
 * -		I2CM_RX_IDLE : receive is idle
 * -		I2CM_RX_BUSY : receive is busy
 * @date	2009/12/25
 */
unsigned short I2CM_checkReceiveBusy(void)
{
	// receive busy flag check
	return	(unsigned short)((RegRead(R16_I2CM_CTL) & RBUSY) >> RBUSY_SFT);
}

/**
 * @brief	get a receive data.
 * - 1. get a receive data and set to "data".
 *
 * @param	*data; unsigned char:			receive data
 * -		from 0x00 to 0xFF
 * @return	none
 * @date	2009/12/25
 */
void I2CM_getReceiveData(unsigned char *data)
{
	// receive data get
	*data = (unsigned char)((RegRead(R16_I2CM_DAT) & RTDT) >> RTDT_SFT);
}

/**
 * @brief	check a receive data register.
 * - 1. check a receive data register condition.
 *
 * @return	unsigned short:					receive data register condition
 * -		I2CM_RXREG_EMPTY    : receive data register is empty
 * -		I2CM_RXREG_NOTEMPTY : receive data register is not empty
 * @date	2009/12/25
 */
unsigned short I2CM_checkReceiveReg(void)
{
	// receive data register check
	return	(unsigned short)((RegRead(R16_I2CM_DAT) & RBRDY) >> RBRDY_SFT);
}

/**
 * @brief	I2C master interrupt initialize.
 * - 1. check parameters.
 * - 2. set an interrupt level.
 *
 * @param	intLevel; unsigned char:		interrupt level
 * -		I2CM_INT_LEVEL0 : level0
 * -		I2CM_INT_LEVEL1 : level1
 * -		I2CM_INT_LEVEL2 : level2
 * -		I2CM_INT_LEVEL3 : level3
 * -		I2CM_INT_LEVEL4 : level4
 * -		I2CM_INT_LEVEL5 : level5
 * -		I2CM_INT_LEVEL6 : level6
 * -		I2CM_INT_LEVEL7 : level7
 * @return	short:							end status
 * -		I2CM_SUCCESS : success
 * -		I2CM_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short I2CM_initInt(unsigned char intLevel)
{
	// parameter check
	if(intLevel >= I2CM_INT_LEVELMAX) {
			return	I2CM_INVALID_PARAMETER;
	}
	
	// set interrupt level
	RegWrite(I2CM_INTLVL, intLevel);
	
	return	I2CM_SUCCESS;
}

/**
 * @brief	I2C master interrupt enable / disable.
 * - 1. check parameters.
 * - 2. set an interrupt enable/disable.
 *
 * @param	selectBit; unsigned short:		interrupt enable / disable
 * -		I2CM_RXINT_DISABLE : receive buffer full interrupt disable
 * -		I2CM_RXINT_ENABLE  : receive buffer full interrupt enable
 * -		I2CM_TXINT_DISABLE : transmit buffer empty interrupt disable
 * -		I2CM_TXINT_ENABLE  : transmit buffer empty interrupt enable
 * @return	short:							end status
 * -		I2CM_SUCCESS : success
 * -		I2CM_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short I2CM_controlInt(unsigned short selectBit)
{
	// parameter check
	if(selectBit > I2CM_IEN_MASK) {
			return	I2CM_INVALID_PARAMETER;
	}
	
	// set interrupt enable / disable
	RegWrite(R16_I2CM_ICTL, selectBit);
	
	return	I2CM_SUCCESS;
}

/**
 * @brief	slave address transmit data generation.
 * - 1. check parameters.
 * - 2. add transfer direction to the slave addresses of 7-bit/10-bit and set the transmit data to "address".
 *
 * @param	selectBit; unsigned char:		select slave address mode
 * -		I2CM_ADDRESS_7BIT  :  7-bit address
 * -		I2CM_ADDRESS_10BIT : 10-bit address
 * @param	dir; unsigned short:			select direction
 * -		I2CM_DIR_TX : data transmit
 * -		I2CM_DIR_RX : data receive
 * @param	*address; unsigned short:		slave address
 * -		 7-bit address : from 0x00 to 0x7F
 * -		10-bit address : from 0x00 to 0xFF
 * @return	short:							end status
 * -		I2CM_SUCCESS : success
 * -		I2CM_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short I2CM_transmitSlaveAddress(unsigned char selectBit, unsigned short dir, unsigned short *address)
{
	unsigned short	work;
	
	// parameter check
	if(	(selectBit > I2CM_ADDRESS_10BIT) ||
		(dir > I2CM_DIR_RX) ) {
			return	I2CM_INVALID_PARAMETER;
	}
	if(	((selectBit == I2CM_ADDRESS_7BIT) && (*address > SLV_07BIT_ADDRESS)) ||
		((selectBit == I2CM_ADDRESS_10BIT) && (*address > SLV_10BIT_ADDRESS)) ) {
			return	I2CM_INVALID_PARAMETER;
	}
	
	// slave address transmit data generation
	if(selectBit == I2CM_ADDRESS_7BIT) {									// 7-bit address mode
		*address <<= 1;
	} else {																// 10-bit address mode
		work = *address;
		*address = ((work & SLV_A7A0) << 8) | ((work & SLV_A9A8) >> 7) | SLV_10BIT_MODE;
	}
	
	// direction set
	*address |= dir;	
	
	return	I2CM_SUCCESS;
}

#endif	// end of PE_I2CM
