/**
 * @file	swt.c
 * @brief	stopwatch timer driver
 * 
 * - Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_SWT
#include <swt_api.h>

/*
 * definition of "define"
 */


/*
 * definition of constants
 */


/*
 * Declarations of function prototype
 */
void SWT_resetTimer(void);
short SWT_setTimerRun(unsigned char selectBit);
short SWT_getCounterDataBCD(unsigned char *data, unsigned char number);
short SWT_initInt(unsigned char intLevel);
short SWT_controlInt(unsigned char selectBit);
short SWT_resetIntFlag(unsigned char selectBit);
void SWT_checkIntFlag (unsigned char *flag);


/**
 * @brief	reset a stopwatch timer.
 * - 1. reset a stopwatch timer.
 *
 * @return	none
 * @date	2009/12/25
 */
void SWT_resetTimer(void)
{
	// stopwatch timer reset
	RegModify(R08_SWT_CTL, SWTRST, (SWTRST * SWT_RESET));
}

/**
 * @brief	set a stopwatch timer run / stop.
 * - 1. check parameters.
 * - 2. set a stopwatch timer run/stop.
 *
 * @param	selectBit; unsigned char: 		select run / stop
 * -		SWT_TIMER_STOP : stopwatch timer stop
 * -		SWT_TIMER_RUN  : stopwatch timer run
 * @return	short:							end status
 * -		SWT_SUCCESS : success
 * -		SWT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short SWT_setTimerRun(unsigned char selectBit)
{
	// parameter check
	if(selectBit > SWT_TIMER_RUN) {
		return	SWT_INVALID_PARAMETER;
	}

	// stopwatch timer run / stop
	RegModify(R08_SWT_CTL, SWTRUN, (selectBit << SWTRUN_SFT));
	
	return	SWT_SUCCESS;
}

/**
 * @brief	get a BCD counter data.
 * - 1. get a counter data from register and set the data to "data".
 * 	-	Note: The correct counter value may not be read out (reading is unstable)
 * 			  if the counter register is read while counting is underway.
 * 	-		  a count value returns success with a continuation match twice
 * 			  and return a failure with a designation number not match. 
 *
 * @param	*data; unsigned char:			counter data
 * -		SWT_DATA_BCD10  :  1/10 second BCD counter data
 * -		SWT_DATA_BCD100 : 1/100 second BCD counter data
 * @param	number; unsigned char:			number of a get count value
 * -		from 0x00 to 0xFF
 * @return	short:							end status
 * -		SWT_SUCCESS : success
 * -		SWT_FAILURE : get count value failure
 * @date	2009/12/25
 */
short SWT_getCounterDataBCD(unsigned char *data, unsigned char number)
{
	unsigned char	i;														// counter
	unsigned char	cmp_data;												// compare data
	
	// BCD counter data get
	for(i = 0; i < number; i++) {
		*data    = RegRead(R08_SWT_BCNT);									// data read
		cmp_data = RegRead(R08_SWT_BCNT);									// compare data read
		
		if(*data == cmp_data) {
			break;															// data OK
		}
	}
	
	// data read check
	if(i == number) {
		return	SWT_FAILURE;
	}
	return	SWT_SUCCESS;
}

/**
 * @brief	stopwatch timer interrupt initialize.
 * - 1. check parameters.
 * - 2. set an interrupt level.
 *
 * @param	intLevel; unsigned char:		interrupt level
 * -		SWT_INT_LEVEL0 : level0
 * -		SWT_INT_LEVEL1 : level1
 * -		SWT_INT_LEVEL2 : level2
 * -		SWT_INT_LEVEL3 : level3
 * -		SWT_INT_LEVEL4 : level4
 * -		SWT_INT_LEVEL5 : level5
 * -		SWT_INT_LEVEL6 : level6
 * -		SWT_INT_LEVEL7 : level7
 * @return	short:							end status
 * -		SWT_SUCCESS : success
 * -		SWT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short SWT_initInt(unsigned char intLevel)
{
	// parameter check
	if(intLevel >= SWT_INT_LEVELMAX) {
		return	SWT_INVALID_PARAMETER;
	}
	
	// set interrupt level
	RegWrite(SWT_INTLVL, intLevel);
	
	return	SWT_SUCCESS;
}

/**
 * @brief	stopwatch timer interrupt enable / disable.
 * - 1. check parameters.
 * - 2. reset an interrupt status when enable interrupt.
 * - 3. set an interrupt enable/disable.
 *
 * @param	selectBit; unsigned char:		interrupt enable / disable
 * -		SWT_1INT_DISABLE   :   1Hz signal interrupt disable
 * -		SWT_1INT_ENABLE    :   1Hz signal interrupt enable
 * -		SWT_10INT_DISABLE  :  10Hz signal interrupt disable
 * -		SWT_10INT_ENABLE   :  10Hz signal interrupt enable
 * -		SWT_100INT_DISABLE : 100Hz signal interrupt disable
 * -		SWT_100INT_ENABLE  : 100Hz signal interrupt enable
 * @return	short:							end status
 * -		SWT_SUCCESS : success
 * -		SWT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short SWT_controlInt(unsigned char selectBit)
{
	// parameter check
	if((selectBit & ~SWT_INT_MASK) != 0) {
		return	SWT_INVALID_PARAMETER;
	}
	
	// reset interrupt flag
	SWT_resetIntFlag(selectBit);
	
	// set interrupt enable
	RegWrite(R08_SWT_IMSK, selectBit);
	
	return	SWT_SUCCESS;
}

/**
 * @brief	stopwatch timer interrupt flag reset.
 * - 1. check parameters.
 * - 2. reset an interrupt status.
 *
 * @param	selectBit; unsigned char:		reset interrupt flag
 * -		SWT_1INT_RESET   :   1Hz signal interrupt flag reset
 * -		SWT_10INT_RESET  :  10Hz signal interrupt flag reset
 * -		SWT_100INT_RESET : 100Hz signal interrupt flag reset
 * @return	short:							end status
 * -		SWT_SUCCESS : success
 * -		SWT_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short SWT_resetIntFlag(unsigned char selectBit)
{
	// parameter check
	if((selectBit & ~SWT_INT_MASK) != 0) {
		return	SWT_INVALID_PARAMETER;
	}
	
	// reset interrupt flag
	RegWrite(R08_SWT_IFLG, selectBit);
	
	return	SWT_SUCCESS;
}

/**
 * @brief	stopwatch timer interrupt flag check.
 * - 1. read an interrupt status and set the status to "flag".
 * 
 * @param	*flag; unsigned char:			check interrupt flag
 * -		SWT_1INT_NOTOCCURRED   :   1Hz signal interrupt is not occurred
 * -		SWT_1INT_OCCURRED      :   1Hz signal interrupt is occurred
 * -		SWT_10INT_NOTOCCURRED  :  10Hz signal interrupt is not occurred
 * -		SWT_10INT_OCCURRED     :  10Hz signal interrupt is occurred
 * -		SWT_100INT_NOTOCCURRED : 100Hz signal interrupt is not occurred
 * -		SWT_100INT_OCCURRED    : 100Hz signal interrupt is occurred
 * @return	none
 * @date	2009/12/25
 */
void SWT_checkIntFlag (unsigned char *flag)
{
	// get interrupt flag
	*flag = RegRead(R08_SWT_IFLG);
}

#endif	// end of PE_SWT
