/**
 * @file	t16.c
 * @brief	16-bit timer driver
 *
 * - Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_T16
#include <t16_api.h>
#include <mux_api.h>

/*
 * definition of "define"
 */


/*
 * definition of constants
 */
static volatile unsigned char	*T16_INTLVL_REG[] = {
	T16_INTLVL_CH0,															// Ch. 0
	T16_INTLVL_CH1,															// Ch. 1
	T16_INTLVL_CH2,															// Ch. 2
#ifndef	T16_CHANNEL3
	T16_INTLVL_CH3,															// Ch. 3
#ifndef	T16_CHANNEL4
	T16_INTLVL_CH4,															// Ch. 4
#endif	// T16_CHANNEL4
#endif	// T16_CHANNEL3
};

/*
 * Declarations of function prototype
 */
short T16_init(unsigned char chNo, unsigned short selectBit, T16_TIM_MODE *mode, unsigned short data);
short T16_setInputClock(unsigned char chNo, unsigned short selectBit);
short T16_setReloadData(unsigned char chNo, unsigned short data);
short T16_getCounterData(unsigned char chNo, unsigned short *data);
short T16_setTimerMode(unsigned char chNo, T16_TIM_MODE *mode);
short T16_resetTimer(unsigned char chNo);
short T16_setTimerRun(unsigned char chNo, unsigned short selectBit);
short T16_initInt(unsigned char chNo, unsigned char intLevel);
short T16_controlInt(unsigned char chNo, unsigned short selectBit);
short T16_resetIntFlag(unsigned char chNo);
short T16_checkIntFlag(unsigned char chNo, unsigned short *flag);


/**
 * @brief	16-bit timer initialize.
 * - 1. check parameters.
 * - 2. stop a 16-bit timer.
 * - 3. set a timer mode.
 * - 4. set a count mode.
 * - 5. set a count clock.(it is not needed for a external clock mode.)
 * - 6. set a polarity of signals.(it is not needed for a internal clock mode.)
 * - 7. set a reload count.
 * - 8. reset a counter.
 *
 * @param	chNo; unsigned char: 			channel number
 * -		T16_CH0 : channel 0
 * -		T16_CH1 : channel 1
 * -		T16_CH2 : channel 2
 * @param	selectBit; unsigned short:		select timer input clock
 * -		refer to T16_setInputClock
 * @param	*mode; T16_TIM_MODE:			mode set parameter
 * -		refer to T16_setTimerMode
 * @param	data; unsigned short:			counter initial value
 * -		from 0x0000 to 0xFFFF
 * @return	short:							end status
 * -		T16_SUCCESS : success
 * -		T16_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16_init(unsigned char chNo, unsigned short selectBit, T16_TIM_MODE *mode, unsigned short data)
{
	// parameter check
	if(	(chNo >= T16_CH_MAX) ||
		(selectBit >= T16_CNTCLK_MAX) ||
		(mode->ckactv > T16_TIMMODE_ACTVH) ||
		(mode->cksl >= T16_TIMMODE_MAX) ||
		(mode->trmd > T16_TIMMODE_ONESHOT) ) {
			return	T16_INVALID_PARAMETER;
	}
	
	// 16-bit timer stop & mode set
	T16_setTimerMode(chNo, mode);

	// 16-bit timer input clock set
	if(mode->cksl != T16_TIMMODE_EXTERNAL) {								// not external clock
		T16_setInputClock(chNo, selectBit);
	}

	// 16-bit timer reload data set
	T16_setReloadData(chNo, data);
	
	// 16-bit timer reset
	T16_resetTimer(chNo);
	
	return	T16_SUCCESS;
}

/**
 * @brief	set a timer input clock.
 * - 1. check parameters.
 * - 2. set a timer input clock.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16_CH0 : channel 0
 * -		T16_CH1 : channel 1
 * -		T16_CH2 : channel 2
 * @param	selectBit; unsigned short:		select timer input clock
 * -		T16_CNTCLK_1     : PCLK 1/1
 * -		T16_CNTCLK_2     : PCLK 1/2
 * -		T16_CNTCLK_4     : PCLK 1/4
 * -		T16_CNTCLK_8     : PCLK 1/8
 * -		T16_CNTCLK_16    : PCLK 1/16
 * -		T16_CNTCLK_32    : PCLK 1/32
 * -		T16_CNTCLK_64    : PCLK 1/64
 * -		T16_CNTCLK_128   : PCLK 1/128
 * -		T16_CNTCLK_256   : PCLK 1/256
 * -		T16_CNTCLK_512   : PCLK 1/512
 * -		T16_CNTCLK_1024  : PCLK 1/1024
 * -		T16_CNTCLK_2048  : PCLK 1/2048
 * -		T16_CNTCLK_4096  : PCLK 1/4096
 * -		T16_CNTCLK_8192  : PCLK 1/8192
 * -		T16_CNTCLK_16384 : PCLK 1/16384
 * @return	short:							end status
 * -		T16_SUCCESS : success
 * -		T16_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16_setInputClock(unsigned char chNo, unsigned short selectBit)
{
	// parameter check
	if(	(chNo >= T16_CH_MAX) ||
		(selectBit >= T16_CNTCLK_MAX) ) {
			return	T16_INVALID_PARAMETER;
	}
	
	// clock stop
	T16_setTimerRun(chNo, T16_TIMER_STOP);
	
	// set count clock
	RegWrite(R16_T16_CLK(chNo), selectBit);
	
	return	T16_SUCCESS;
}

/**
 * @brief	set a 16-bit timer reload data.
 * - 1. check parameters.
 * - 2. set a counter initial value of 16-bit timer.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16_CH0 : channel 0
 * -		T16_CH1 : channel 1
 * -		T16_CH2 : channel 2
 * @param	data; unsigned short:			counter initial value
 * -		from 0x0001 to 0xFFFF
 * @return	short:							end status
 * -		T16_SUCCESS : success
 * -		T16_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16_setReloadData(unsigned char chNo, unsigned short data)
{
	// parameter check
	if(chNo >= T16_CH_MAX) {
		return	T16_INVALID_PARAMETER;
	}
	
	// set counter value
	RegWrite(R16_T16_TR(chNo), data);
	
	return	T16_SUCCESS;
}

/**
 * @brief	get a counter data.
 * - 1. check parameters.
 * - 2. get a counter data from register and set the data to "data".
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16_CH0 : channel 0
 * -		T16_CH1 : channel 1
 * -		T16_CH2 : channel 2
 * @param	*data; unsigned short:			counter data
 * -		from 0x0000 to 0xFFFF
 * @return	short:							end status
 * -		T16_SUCCESS : success
 * -		T16_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16_getCounterData(unsigned char chNo, unsigned short *data)
{
	// parameter check
	if(chNo >= T16_CH_MAX) {
		return	T16_INVALID_PARAMETER;
	}
	
	// get counter data
	*data = RegRead(R16_T16_TC(chNo));
	
	return	T16_SUCCESS;
}

/**
 * @brief	set a 16-bit timer mode.
 * - 1. check parameters.
 * - 2. set a 16-bit timer according to parameters.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16_CH0 : channel 0
 * -		T16_CH1 : channel 1
 * -		T16_CH2 : channel 2
 * @param	*mode; T16_TIM_MODE:			mode set parameter
 * -		ckactv; unsigned short:			select active polarity/select count edge
 * 	-			T16_TIMMODE_ACTVL : active low/negative edge
 * 	-			T16_TIMMODE_ACTVH : active high/positive edge
 * -		cksl; unsigned short:			select operational mode
 * 	-			T16_TIMMODE_INTERNAL : internal clock mode
 * 	-			T16_TIMMODE_EXTERNAL : external clock mode
 * 	-			T16_TIMMODE_PULS     : pulse width measurement mode
 * -		trmd; unsigned short:			select count mode
 * 	-			T16_TIMMODE_REPEAT  : repeat mode
 * 	-			T16_TIMMODE_ONESHOT : one shot mode
 * @return	short:							end status
 * -		T16_SUCCESS : success
 * -		T16_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16_setTimerMode(unsigned char chNo, T16_TIM_MODE *mode)
{
	// parameter check
	if( (chNo >= T16_CH_MAX) ||
		(mode->ckactv > T16_TIMMODE_ACTVH) ||
		(mode->cksl >= T16_TIMMODE_MAX) ||
		(mode->trmd > T16_TIMMODE_ONESHOT) ) {
			return	T16_INVALID_PARAMETER;
	}

	// input port set for external mode or puls width measurement mode
	if(mode->cksl != T16_TIMMODE_INTERNAL) {
		MUX_setT16port(chNo, MUX_ON);
	}

	// clock stop
	T16_setTimerRun(chNo, T16_TIMER_STOP);
	
	// set timer mode
	RegModify(R16_T16_CTL(chNo), (CKACTV | CKSL |TRMD),
		((mode->ckactv << CKACTV_SFT) | (mode->cksl << CKSL_SFT) | (mode->trmd << TRMD_SFT)));

	return	T16_SUCCESS;
}

/**
 * @brief	reset a 16-bit timer.
 * - 1. check parameters.
 * - 2. reset a 16-bit timer.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16_CH0 : channel 0
 * -		T16_CH1 : channel 1
 * -		T16_CH2 : channel 2
 * @return	short:							end status
 * -		T16_SUCCESS : success
 * -		T16_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16_resetTimer(unsigned char chNo)
{
	// parameter check
	if(chNo >= T16_CH_MAX) {
		return	T16_INVALID_PARAMETER;
	}
	
	// 16-bti timer reset
	RegModify(R16_T16_CTL(chNo), PRESER, (PRESER * T16_RESET));
	
	return	T16_SUCCESS;
}

/**
 * @brief	set a 16-bit timer run / stop.
 * - 1. check parameters.
 * - 2. set a 16-bit timer run/stop.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16_CH0 : channel 0
 * -		T16_CH1 : channel 1
 * -		T16_CH2 : channel 2
 * @param	selectBit; unsigned short: 		select run / stop
 * -		T16_TIMER_STOP : 16-bit timer stop
 * -		T16_TIMER_RUN  : 16-bit timer run
 * @return	short:							end status
 * -		T16_SUCCESS : success
 * -		T16_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16_setTimerRun(unsigned char chNo, unsigned short selectBit)
{
	// parameter check
	if(	(chNo >= T16_CH_MAX) ||
		(selectBit > T16_TIMER_RUN) ) {
			return	T16_INVALID_PARAMETER;
	}
	
	// 16-bit timer run / stop
	RegModify(R16_T16_CTL(chNo), PRUN, (selectBit << PRUN_SFT));
	
	return	T16_SUCCESS;
}

/**
 * @brief	16-bit timer interrupt initialize.
 * - 1. check parameters.
 * - 2. set an interrupt level.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16_CH0 : channel 0
 * -		T16_CH1 : channel 1
 * -		T16_CH2 : channel 2
 * @param	intLevel; unsigned char:		interrupt level
 * -		T16_INT_LEVEL0 : level0
 * -		T16_INT_LEVEL1 : level1
 * -		T16_INT_LEVEL2 : level2
 * -		T16_INT_LEVEL3 : level3
 * -		T16_INT_LEVEL4 : level4
 * -		T16_INT_LEVEL5 : level5
 * -		T16_INT_LEVEL6 : level6
 * -		T16_INT_LEVEL7 : level7
 * @return	short:							end status
 * -		T16_SUCCESS : success
 * -		T16_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16_initInt(unsigned char chNo, unsigned char intLevel)
{
	// parameter check
	if( (chNo >= T16_CH_MAX) ||
		(intLevel >= T16_INT_LEVELMAX) ) {
			return	T16_INVALID_PARAMETER;
	}
	
	// set interrupt level
	RegWrite(T16_INTLVL_REG[chNo], intLevel);
	
	return	T16_SUCCESS;
}

/**
 * @brief	16-bit timer interrupt enable / disable.
 * - 1. check parameters.
 * - 2. reset an interrupt status when enable interrupt.
 * - 3. set an interrupt enable/disable.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16_CH0 : channel 0
 * -		T16_CH1 : channel 1
 * -		T16_CH2 : channel 2
 * @param	selectBit; unsigned short:		interrupt enable / disable
 * -		T16_INT_DISABLE : interrupt disable
 * -		T16_INT_ENABLE  : interrupt enable
 * @return	short:							end status
 * -		T16_SUCCESS : success
 * -		T16_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16_controlInt(unsigned char chNo, unsigned short selectBit)
{
	// parameter check
	if(	(chNo >= T16_CH_MAX) ||
		(selectBit > T16_INT_ENABLE) ) {
			return	T16_INVALID_PARAMETER;
	}
	
	// reset interrupt flag
	T16_resetIntFlag(chNo);
	
	// set interrupt enable
	RegModify(R16_T16_INT(chNo), T16IE, (selectBit << T16IE_SFT));
	
	return	T16_SUCCESS;
}

/**
 * @brief	16-bit timer interrupt flag reset.
 * - 1. check parameters.
 * - 2. reset an interrupt status.
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16_CH0 : channel 0
 * -		T16_CH1 : channel 1
 * -		T16_CH2 : channel 2
 * @return	short:							end status
 * -		T16_SUCCESS : success
 * -		T16_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16_resetIntFlag(unsigned char chNo)
{
	// parameter check
	if(chNo >= T16_CH_MAX) {
		return	T16_INVALID_PARAMETER;
	}
	
	// reset interrupt flag
	RegModify(R16_T16_INT(chNo), T16IF, (T16IF * T16_RESET));
	
	return	T16_SUCCESS;
}

/**
 * @brief	16-bit timer interrupt flag check.
 * - 1. check parameters.
 * - 2. read an interrupt status and set the status to "flag".
 *
 * @param	chNo; unsigned char:			channel number
 * -		T16_CH0 : channel 0
 * -		T16_CH1 : channel 1
 * -		T16_CH2 : channel 2
 * @param	*flag; unsigned short:			check interrupt flag
 * -		T16_TIMERINT_NOTOCCURRED : interrupt is not occurred
 * -		T16_TIMERINT_OCCURRED    : interrupt is occurred
 * @return	short:							end status
 * -		T16_SUCCESS : success
 * -		T16_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short T16_checkIntFlag(unsigned char chNo, unsigned short *flag)
{
	// parameter check
	if(chNo >= T16_CH_MAX) {
		return	T16_INVALID_PARAMETER;
	}
	
	// get interrupt flag
	*flag = (RegRead(R16_T16_INT(chNo)) & T16IF) >> T16IF_SFT;
	
	return	T16_SUCCESS;
}

#endif	// end of PE_T16
