/**
 * @file	uart.c
 * @brief	UART driver
 *
 * - Copyright(C) SEIKO EPSON CORPORATION 2010. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_UART
#include <uart_api.h>
#include <mux_api.h>

/*
 * definition of "define"
 */


/*
 * definition of constants
 */
static volatile unsigned char	*UART_INTLVL_REG[] = {
	UART_INTLVL_CH0,														// Ch. 0
#ifdef	UART_CHANNEL2
	UART_INTLVL_CH1															// Ch. 1
#endif	// end of UART_CHANNEL2
};


/*
 * Declarations of function prototype
 */
short UART_init(unsigned char chNo, UART_INIT_PARAM *param);
short UART_setTransmitData(unsigned char chNo, unsigned char data);
short UART_getReceiveData(unsigned char chNo, unsigned char *data);
short UART_setComEnable(unsigned char chNo, unsigned char selectBit);
short UART_initInt(unsigned char chNo, unsigned char intLevel);
short UART_controlInt(unsigned char chNo, unsigned char selectBit, unsigned char condition);
short UART_resetIntFlag(unsigned char chNo, unsigned char selectBit);
short UART_checkReceiveFlag(unsigned char chNo, unsigned char *flag);
short UART_setIrDAmode(unsigned char chNo, unsigned char selectClk, unsigned char selectBit);


/**
 * @brief	UART initialize.
 * - 1. check parameters.
 * - 2. stop a TX/RX.
 * - 3. set UART ports.(SCLK-pin is needed for external clock mode.)
 * - 4. set a transfer data length.
 * - 5. set a parity.
 * - 6. set a stop bit length.
 * - 7. set an input clock.
 *
 * @param	chNo; unsigned char: 			channel number
 * -		UART_CH0 : channel 0
 * -		UART_CH1 : channel 1
 * @param	*param; UART_INIT_PARAM:		UART initialize parameter
 * -		dataLen; unsigned char:			select transfer data length
 * 	-			UART_DATALEN_7BIT : 7-bit
 * 	-			UART_DATALEN_8BIT : 8-bit
 * -		parity; unsigned char:			select parity
 * 	-			UART_PARITY_EVEN : even parity
 * 	-			UART_PARITY_ODD  : odd parigy
 * 	-			UART_PARITY_NONE : none parity
 * -		stopBit; unsigned char:			select stop bit length
 * 	-			UART_STOPBIT1 : 1-bit
 * 	-			UART_STOPBIT2 : 2-bit
 * -		inputClk; unsigned char:		select input clock
 * 	-			UART_CLK_INTERNAL : internal clock
 * 	-			UART_CLK_EXTERNAL : external clock
 * @return	short:							end status
 * -		UART_SUCCESS : success
 * -		UART_INVALID_PARAMETER : invalid parameter
 * @date	2010/04/12 : Fix MUX_setUARTport
 * @version
 * -		2009/12/25
 */
short UART_init(unsigned char chNo, UART_INIT_PARAM *param)
{
	MUX_USE_UARTPORT 	use;
	// parameter check
	if(	(chNo >= UART_CH_MAX) ||
		(param->dataLen > UART_DATALEN_8BIT) ||
		(param->parity > UART_PARITY_ODD) ||
		(param->stopBit > UART_STOPBIT2) ||
		(param->inputClk > UART_CLK_EXTERNAL) ) {
			return	UART_INVALID_PARAMETER;
	}

	// UART disable
	UART_setComEnable(chNo, UART_CTL_DISABLE);

	// set UART port
#if	defined(S1C17624) || defined(S1C17622)
	if (chNo == UART_CH0) {
		use.selectNo = MUX_UART_CH0;
	} else {
		use.selectNo = MUX_UART_CH1_0;
	}
#endif	// end of S1C17624/S1C17622
	use.sw = MUX_ON;
	MUX_setUARTport(chNo, param->inputClk, &use);

	// set initialize parameter
	RegWrite(R08_UART_MOD(chNo),
		((param->dataLen << CHLN_SFT) | (param->parity << PMD_SFT) | (param->stopBit << STPB_SFT) | (param->inputClk << SSCK_SFT)));

	return	UART_SUCCESS;
}

/**
 * @brief	set a transmit data.
 * - 1. check parameters.
 * - 2. set a transmit data.
 *
 * @param	chNo; unsigned char: 			channel number
 * -		UART_CH0 : channel 0
 * -		UART_CH1 : channel 1
 * @param	data; unsigned char:			transmit data
 * -		from 0x00 to 0xFF(0x7F at the time of 7-bit mode)
 * @return	short:							end status
 * -		UART_SUCCESS : success
 * -		UART_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short UART_setTransmitData(unsigned char chNo, unsigned char data)
{
	// parameter check
	if(chNo >= UART_CH_MAX) {
		return	UART_INVALID_PARAMETER;
	}

	// set transmit data
	RegWrite(R08_UART_TXD(chNo), data);

	return	UART_SUCCESS;
}

/**
 * @brief	get a receive data.
 * - 1. check parameters.
 * - 2. get a receive data and set to "data".
 *
 * @param	chNo; unsigned char: 			channel number
 * -		UART_CH0 : channel 0
 * -		UART_CH1 : channel 1
 * @param	*data; unsigned char:			receive data
 * -		from 0x00 to 0xFF(0x7F at the time of 7-bit mode)
 * @return	short:							end status
 * -		UART_SUCCESS : success
 * -		UART_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short UART_getReceiveData(unsigned char chNo, unsigned char *data)
{
	// parameter check
	if(chNo >= UART_CH_MAX) {
		return	UART_INVALID_PARAMETER;
	}

	// get receive data
	*data = RegRead(R08_UART_RXD(chNo));

	return	UART_SUCCESS;
}

/**
 * @brief	set an UART enable / disable.
 * - 1. check parameters.
 * - 2. set an UART enable/disable.
 *
 * @param	chNo; unsigned char: 			channel number
 * -		UART_CH0 : channel 0
 * -		UART_CH1 : channel 1
 * @param	selectBit; unsigned char:		UART enable / disable
 * -		UART_CTL_DISABLE : disable
 * -		UART_CTL_ENABLE  : enable
 * @return	short:							end status
 * -		UART_SUCCESS : success
 * -		UART_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short UART_setComEnable(unsigned char chNo, unsigned char selectBit)
{
	// parameter check
	if(	(chNo >= UART_CH_MAX) ||
		(selectBit > UART_ENABLE) ) {
			return	UART_INVALID_PARAMETER;
	}

	// set UART enable / disable
	RegModify(R08_UART_CTL(chNo), RXEN, (selectBit << RXEN_SFT));

	return	UART_SUCCESS;
}

/**
 * @brief	UART interrupt initialize.
 * - 1. check parameters.
 * - 2. set an interrupt level.
 *
 * @param	chNo; unsigned char:			channel number
 * -		UART_CH0 : channel 0
 * -		UART_CH1 : channel 1
 * @param	intLevel; unsigned char:		interrupt level
 * -		UART_INT_LEVEL0 : level0
 * -		UART_INT_LEVEL1 : level1
 * -		UART_INT_LEVEL2 : level2
 * -		UART_INT_LEVEL3 : level3
 * -		UART_INT_LEVEL4 : level4
 * -		UART_INT_LEVEL5 : level5
 * -		UART_INT_LEVEL6 : level6
 * -		UART_INT_LEVEL7 : level7
 * @return	short:							end status
 * -		UART_SUCCESS : success
 * -		UART_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short UART_initInt(unsigned char chNo, unsigned char intLevel)
{
	// parameter check
	if(	(chNo >= UART_CH_MAX) ||
		(intLevel >= UART_INT_LEVELMAX) ) {
			return	UART_INVALID_PARAMETER;
	}

	// set interrupt level
	RegWrite(UART_INTLVL_REG[chNo], intLevel);

	return	UART_SUCCESS;
}

/**
 * @brief	UART interrupt enable / disable.
 * - 1. check parameters.
 * - 2. reset an interrupt status when enable interrupt.
 * - 3. set an interrupt enable/disable.
 * - 4. set a receive buffer full interrupt condition.
 *
 * @param	chNo; unsigned char:			channel number
 * -		UART_CH0 : channel 0
 * -		UART_CH1 : channel 1
 * @param	selectBit; unsigned char:		interrupt enable / disable
 * -		UART_RXERRORINT_DISABLE : receiving error interrupt disable
 * -		UART_RXERRORINT_ENABLE  : receiving error interrupt enable
 * -		UART_RXINT_DISABLE      : receive buffer full interrupt disable
 * -		UART_RXINT_ENABLE       : receive buffer full interrupt enable
 * -		UART_TXINT_DISABLE      : transmit buffer empty interrupt disable
 * -		UART_TXINT_ENABLE       : transmit buffer empty interrupt enable
 * @param	condition; unsigned char:		receive buffer full interrupt condition
 * -		UART_RXINT_1BYTE : 1-byte
 * -		UART_RXINT_2BYTE : 2-byte
 * @return	short:							end status
 * -		UART_SUCCESS : success
 * -		UART_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short UART_controlInt(unsigned char chNo, unsigned char selectBit, unsigned char condition)
{
	// parameter check
	if(	(chNo >= UART_CH_MAX) ||
		((selectBit & ~UART_IEN_MASK ) != 0) ||
		(condition > UART_2BIT) ) {
			return	UART_INVALID_PARAMETER;
	}

	// reset interrupt flag
	if((selectBit & REIEN) != 0) {
		UART_resetIntFlag(chNo, (UART_FRAMING_RESET | UART_PARITY_RESET | UART_OVERRUN_RESET));
	}

	// set interrupt enable / disable
	RegModify(R08_UART_CTL(chNo), UART_IEN_MASK, selectBit);

	// set receive buffer full interrupt condition
	RegModify(R08_UART_CTL(chNo), RBFI, (condition << RBFI_SFT));

	return	UART_SUCCESS;
}

/**
 * @brief	UART interrupt flag reset.
 * - 1. check parameters.
 * - 2. reset an interrupt status.
 *
 * @param	chNo; unsigned char:			channel number
 * -		UART_CH0 : channel 0
 * -		UART_CH1 : channel 1
 * @param	selectBit; unsigned char:		reset interrupt flag
 * -		UART_FRAMING_RESET : framing error interrupt flag reset
 * -		UART_PARITY_RESET  : parity error interrupt flag reset
 * -		UART_OVERRUN_RESET : over run error interrupt flag reset
 * @return	short:							end status
 * -		UART_SUCCESS : success
 * -		UART_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short UART_resetIntFlag(unsigned char chNo, unsigned char selectBit)
{
	// parameter check
	if(	(chNo >= UART_CH_MAX) ||
		((selectBit & ~UART_REIEN_MASK) != 0) ) {
			return	UART_INVALID_PARAMETER;
	}

	// reset interrupt flag
	RegModify(R08_UART_ST(chNo), UART_REIEN_MASK, selectBit);

	return	UART_SUCCESS;
}

/**
 * @brief	UART interrupt flag check.
 * - 1. check parameters.
 * - 2. read an interrupt status and set the status to "flag".
 *
 * @param	chNo; unsigned char:			channel number
 * -		UART_CH0 : channel 0
 * -		UART_CH1 : channel 1
 * @param	*flag; unsigned char:			check interrupt flag
 * -		UART_FRAMING_NOTOCCURRED : framing error interrupt is not occurred
 * -		UART_FRAMING_OCCURRED    : framing error interrupt is occurred
 * -		UART_PARITY_NOTOCCURRED  : parity error interrupt is not occurred
 * -		UART_PARITY_OCCURRED     : parity error interrupt is occurred
 * -		UART_OVERRUN_NOTOCCURRED : over run error interrupt is not occurred
 * -		UART_OVERRUN_OCCURRED    : over run error interrupt is occurred
 * -		UART_2BYTE_EMPTY         : 2-byte unreceiving
 * -		UART_2BYTE_READY         : 2-byte read ready
 * -		UART_TX_IDLE             : transmit shift register idle
 * -		UART_TX_BUSY             : transmit shift register busy
 * -		UART_RXDATA_EMPTY        : receive data buffer empty
 * -		UART_RXDATA_READY        : receive data buffer ready
 * -		UART_TXBUFFER_NOTEMPTY   : transmit data buffer is not empty
 * -		UART_TXBUFFER_EMPTY      : transmit data buffer is empty
 * @return	short:							end status
 * -		UART_SUCCESS : success
 * -		UART_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short UART_checkReceiveFlag(unsigned char chNo, unsigned char *flag)
{
	// parameter check
	if(chNo >= UART_CH_MAX) {
		return	UART_INVALID_PARAMETER;
	}

	// get interrupt flag
	*flag = RegRead(R08_UART_ST(chNo));

	return	UART_SUCCESS;
}

/**
 * @brief	set an IrDA mode.
 * - 1. check parameters.
 * - 2. set an IrDA mode.
 *
 * @param	chNo; unsigned char:			channel number
 * -		UART_CH0 : channel 0
 * -		UART_CH1 : channel 1
 * @param	selectClk; unsigned char:		select clock
 * -		UART_IRDA_PCLK1   : PCLK 1/1
 * -		UART_IRDA_PCLK2   : PCLK 1/2
 * -		UART_IRDA_PCLK4   : PCLK 1/4
 * -		UART_IRDA_PCLK8   : PCLK 1/8
 * -		UART_IRDA_PCLK16  : PCLK 1/16
 * -		UART_IRDA_PCLK32  : PCLK 1/32
 * -		UART_IRDA_PCLK64  : PCLK 1/64
 * -		UART_IRDA_PCLK128 : PCLK 1/128
 * @param	selectBit; unsigned char:		IrDA mode
 * -		UART_IRDA_OFF : IrDA mode off
 * -		UART_IRDA_ON  : IrDA mode on
 * @return	short:							end status
 * -		UART_SUCCESS : success
 * -		UART_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short UART_setIrDAmode(unsigned char chNo, unsigned char selectClk, unsigned char selectBit)
{
	// parameter check
	if(	(chNo >= UART_CH_MAX) ||
		(selectClk >= UART_IRDA_MAX) ||
		(selectBit > UART_IRDA_ON) ) {
			return	UART_INVALID_PARAMETER;
	}

	// set IrDA mode
	RegWrite(R08_UART_EXP(chNo), ((selectClk << IRCLK_SFT) | (selectBit << IRMD_SFT)));

	return	UART_SUCCESS;
}

#endif	// end of PE_UART
